import rclpy
from rclpy.node import Node
from active_bo_msgs.srv import ActiveRL

import time


class ActiveRLProxy(Node):
    def __init__(self):
        super().__init__('active_rl_proxy')

        self.client = self.create_client(ActiveRL, 'arl_proxy')

        while not self.client.wait_for_service(timeout_sec=10.0):
            self.get_logger().info('Waiting for the active_rl_srv')

        self.server = self.create_service(ActiveRL, 'active_rl_srv', self.proxy_callback)

    def proxy_callback(self, request, response):

        self.get_logger().info(f'Received request:')
        self.get_logger().info(f' old_policy: {request.old_policy}')
        self.get_logger().info(f' old_weights: {request.old_weights}')

        # Forward the request to the original service server
        future = self.client.call_async(request)

        timeout = 10
        start_time = time.time()
        while not future.done():
            rclpy.spin_once(self, timeout_sec=0.1)
            self.get_logger().info(f'{future.result()}')
            if time.time() - start_time > timeout:
                self.get_logger().error('Service call timed out.')
                break
        # self.executor.spin_until_future_complete(future)

        if future.result() is not None:
            response.new_weights = future.result().new_weights
            response.final_step = future.result().final_step
            response.reward = future.result().reward

            self.get_logger().info(f'Sending response:')
            self.get_logger().info(f' new_weigths: {response.new_weights}')
            self.get_logger().info(f'  final_step: {response.final_step}')
            self.get_logger().info(f'  reward: {response.reward}')
        # result = self.client.call(request)
        # if result is not None:
        #     response.new_weights = result.new_weights
        #     response.final_step = result.final_step
        #     response.reward = result.reward
        #
        #     self.get_logger().info(f'Sending response:')
        #     self.get_logger().info(f' new_weigths: {response.new_weights}')
        #     self.get_logger().info(f'  final_step: {response.final_step}')
        #     self.get_logger().info(f'  reward: {response.reward}')
        else:
            self.get_logger().error('Failed to call the original service')

        return response


def main(args=None):
    rclpy.init(args=args)
    service_proxy = ActiveRLProxy()

    try:
        rclpy.spin(service_proxy)
    except KeyboardInterrupt:
        pass

    service_proxy.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
