import numpy as np


class GaussianRBF:
    def __init__(self, nr_weights, nr_steps, seed=None, lowerb=-1.0, upperb=1.0):
        self.nr_weights = nr_weights
        self.nr_steps = nr_steps
        self.weights = None
        self.policy = None
        self.mean = np.linspace(0, self.nr_steps, self.nr_weights)
        if nr_weights > 1:
            self.std = self.mean[1] / (2 * np.sqrt(2 * np.log(2)))  # Full width at half maximum
        else:
            self.std = self.nr_steps / 2

        self.rng = np.random.default_rng(seed=seed)
        self.low = lowerb
        self.upper = upperb

        self.reset()

    def reset(self):
        self.weights = np.zeros((self.nr_weights, 1))
        self.policy = np.zeros((self.nr_steps, 1))

    def random_policy(self):
        self.weights = np.around(self.rng.uniform(self.low, self.upper, self.nr_weights), decimals=8)
        return self.weights.T

    def rollout(self):
        self.policy = np.zeros((self.nr_steps, 1))
        for i in range(self.nr_steps):
            for j in range(self.nr_weights):
                base_fun = np.exp(-0.5*(i - self.mean[j])**2 / self.std**2)
                self.policy[i] += base_fun * self.weights[j]

        return self.policy


def main():
    policy = GaussianRBF(1, 50)
    policy.random_policy()
    policy.rollout()
    print(policy.weights)


if __name__ == "__main__":
    main()
