from active_bo_msgs.srv import BO

import rclpy
from rclpy.node import Node

from active_bo_ros.BayesianOptimization.BayesianOptimization import BayesianOptimization
from active_bo_ros.ReinforcementLearning.ContinuousMountainCar import Continuous_MountainCarEnv

import numpy as np


class BOService(Node):
    def __init__(self):
        super().__init__('bo_service')
        self.srv = self.create_service(BO, 'bo_srv', self.bo_callback)

        self.env = Continuous_MountainCarEnv()
        self.distance_penalty = 0

        self.nr_init = 3

    def bo_callback(self, request, response):
        self.get_logger().info('Bayesian Optimization Service started!')
        nr_weights = request.nr_weights
        max_steps = request.max_steps
        nr_episodes = request.nr_episodes
        nr_runs = request.nr_runs
        acq = request.acquisition_function
        self.get_logger().info(acq)

        reward = np.zeros((nr_episodes, nr_runs))
        best_pol_reward = np.zeros((1, nr_runs))
        best_policy = np.zeros((max_steps, nr_runs))
        best_weights = np.zeros((nr_weights, nr_runs))

        BO = BayesianOptimization(self.env,
                                  max_steps,
                                  nr_init=self.nr_init,
                                  acq=acq,
                                  nr_weights=nr_weights)

        for i in range(nr_runs):
            BO.initialize()

            for j in range(nr_episodes):
                x_next = BO.next_observation()
                BO.eval_new_observation(x_next)

            best_policy[:, i], best_pol_reward[:, i], best_weights[:, i] = BO.get_best_result()

            reward[:, i] = BO.best_reward.T

        response.reward_mean = np.mean(reward, axis=1).tolist()
        response.reward_std = np.std(reward, axis=1).tolist()

        best_policy_idx = np.argmax(best_pol_reward)
        response.best_weights = best_weights[:, best_policy_idx].tolist()
        response.best_policy = best_policy[:, best_policy_idx].tolist()
        return response


def main(args=None):
    rclpy.init(args=args)

    bo_service = BOService()

    rclpy.spin(bo_service)


if __name__ == '__main__':
    main()
