from active_bo_msgs.srv import ActiveRL
from active_bo_msgs.srv import ActiveRLEval
from active_bo_msgs.msg import ImageFeedback

import rclpy
from rclpy.node import Node

from active_bo_ros.ReinforcementLearning.ContinuousMountainCar import Continuous_MountainCarEnv

import numpy as np


class ActiveRLService(Node):
    def __init__(self):
        super().__init__('active_rl_service')
        self.srv = self.create_service(ActiveRL, 'active_rl_srv', self.active_rl_callback)
        self.eval_srv = self.create_client(ActiveRLEval, 'active_rl_eval_srv')

        self.publisher = self.create_publisher(ImageFeedback, 'rl_feedback', 1)

        self.env = Continuous_MountainCarEnv(render_mode='rgb_array')
        self.distance_penalty = 0

    def active_rl_callback(self, request, response):

        feedback_msg = ImageFeedback()

        reward = 0
        step_count = 0
        old_policy = request.old_policy
        old_weights = request.old_weights

        eval_request = ActiveRLEval.Request()
        eval_request.old_policy = old_policy
        eval_request.old_weights = old_weights

        self.env.reset()

        self.get_logger().info('Best policy so far!')

        for i in range(len(old_policy)):
            action = old_policy[i]
            output = self.env.step(action)

            done = output[2]

            rgb_array = self.env.render()
            rgb_shape = rgb_array.shape

            red = rgb_array[:, :, 0].flatten().tolist()
            green = rgb_array[:, :, 1].flatten().tolist()
            blue = rgb_array[:, :, 2].flatten().tolist()

            feedback_msg.height = rgb_shape[0]
            feedback_msg.width = rgb_shape[1]
            feedback_msg.red = red
            feedback_msg.green = green
            feedback_msg.blue = blue

            self.publisher.publish(feedback_msg)

            if done:
                break

        self.get_logger().info('Enter new solution!')
        eval_response = self.eval_srv.call(eval_request)
        self.get_logger().info('Service responded!')

        new_policy = eval_response.new_policy

        for i in range(len(new_policy)):
            action = new_policy[i]
            output = self.env.step(action)

            reward += output[1]
            done = output[2]
            step_count += 1

            rgb_array = self.env.render()
            rgb_shape = rgb_array.shape

            red = rgb_array[:, :, 0].flatten().tolist()
            green = rgb_array[:, :, 1].flatten().tolist()
            blue = rgb_array[:, :, 2].flatten().tolist()

            feedback_msg.height = rgb_shape[0]
            feedback_msg.width = rgb_shape[1]
            feedback_msg.red = red
            feedback_msg.green = green
            feedback_msg.blue = blue

            self.publisher.publish(feedback_msg)

            if done:
                break

            if not done and i == len(new_policy):
                distance = -(self.env.goal_position - output[0][0])
                reward += distance * self.distance_penalty

        response.new_weights = eval_response.Response.new_weights
        response.reward = reward
        response.final_step = step_count

        return response


def main(args=None):
    rclpy.init(args=args)

    active_rl_service = ActiveRLService()

    rclpy.spin(active_rl_service)


if __name__ == '__main__':
    main()
