from active_bo_msgs.srv import RLRollOut
from active_bo_msgs.msg import ImageFeedback

import rclpy
from rclpy.node import Node

from active_bo_ros.ReinforcementLearning.ContinuousMountainCar import Continuous_MountainCarEnv
from active_bo_ros.ReinforcementLearning.CartPole import CartPoleEnv
from active_bo_ros.ReinforcementLearning.Pendulum import PendulumEnv
from active_bo_ros.ReinforcementLearning.Acrobot import AcrobotEnv

import numpy as np


class RLService(Node):
    def __init__(self):
        super().__init__('rl_service')
        self.srv = self.create_service(RLRollOut, 'rl_srv', self.rl_callback)

        self.publisher = self.create_publisher(ImageFeedback, 'rl_feedback', 1)

        self.env = None

    def rl_callback(self, request, response):

        feedback_msg = ImageFeedback()

        reward = 0
        step_count = 0
        policy = np.array(request.policy, dtype=np.float64)
        rl_env = request.env

        if rl_env == "Mountain Car":
            self.env = Continuous_MountainCarEnv(render_mode="rgb_array")
        elif rl_env == "Cartpole":
            self.env = CartPoleEnv(render_mode="rgb_array")
        elif rl_env == "Acrobot":
            self.env = AcrobotEnv(render_mode="rgb_array")
        elif rl_env == "Pendulum":
            self.env = PendulumEnv(render_mode="rgb_array")
        else:
            raise NotImplementedError

        self.env.reset()

        for i in range(len(policy)):
            action = policy[i]
            action_clipped = action.clip(min=-1.0, max=1.0)
            output = self.env.step(action_clipped.astype(np.float64))

            reward += output[1]
            done = output[2]
            step_count += 1

            rgb_array = self.env.render()
            rgb_shape = rgb_array.shape

            red = rgb_array[:, :, 0].flatten().tolist()
            green = rgb_array[:, :, 1].flatten().tolist()
            blue = rgb_array[:, :, 2].flatten().tolist()

            feedback_msg.height = rgb_shape[0]
            feedback_msg.width = rgb_shape[1]
            feedback_msg.red = red
            feedback_msg.green = green
            feedback_msg.blue = blue

            self.publisher.publish(feedback_msg)

            if done:
                break

        response.reward = reward
        response.final_step = step_count

        return response


def main(args=None):
    rclpy.init(args=args)

    rl_service = RLService()

    rclpy.spin(rl_service)

    rclpy.shutdown()


if __name__ == '__main__':
    main()
